unit RS232;

interface

type TPBA= word;
     TLink= (CTS, DSR, RI, DCD, DTR, RTS, TxD);

const
     COM1= $3F8;
     COM2= $2F8;
     COM3= $3E8;
     COM4= $3F8;

var  PBA: TPBA;

{ General Port Handle }
function ReadPort(pAddress: Word):byte;
procedure WritePort(pAddress: Word; Data: byte);

{ Inputs }
function Scan_CTS(PA: TPBA): boolean;
function Scan_DSR(PA: TPBA): boolean;
function Scan_RI (PA: TPBA): boolean;
function Scan_DCD(PA: TPBA): boolean;

{ Outputs }
function Scan_DTR(PA: TPBA): boolean;
function Scan_RTS(PA: TPBA): boolean;
function Scan_TxD(PA: TPBA): boolean;

{ Scan User }
function ScanUserLink(Link: TLink): boolean;

{ Links Tabs Allocations }
function GetLinkOffset(Link: TLink): byte;
function GetLinkInNumber(Link: TLink) : byte;

implementation

{ General Port Communications Routines }
function ReadPort(pAddress: Word):byte;
var InReadPort: byte;
begin
  asm
    Mov dx, pAddress
    In  al, dx
    Nop
    Nop
    Mov byte ptr InReadPort, al
  end;
  ReadPort:= InReadPort;
end;

procedure WritePort(pAddress: Word; Data: byte);
begin
  asm
    Mov dx, pAddress
    Mov al, Data
    Out dx, al
  end;
end;

{ Inputs Scan Routines }

function Scan_CTS(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+6) and $10) = $10;
end;

function Scan_DSR(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+6) and $20) = $20;
end;

function Scan_RI (PA: TPBA): boolean;
begin
  result:=  (ReadPort(PA+6) and $40) = $40;
end;

function Scan_DCD(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+6) and $80) = $80;
end;

{ Outputs Scan Routines }

function Scan_DTR(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+4) and $01) = $01;
end;

function Scan_RTS(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+4) and $02) = $02;
end;

function Scan_TxD(PA: TPBA): boolean;
begin
  result:= (ReadPort(PA+3) and $40) = $40;
end;

{ Scan User }
function GetLinkOffset(Link: TLink): byte;
begin
  case Link of
    CTS..DCD: GetLinkOffset:= 6;
    DTR..RTS: GetLinkOffset:= 4;
    TxD     : GetLinkOffset:= 3;
  end;
end;

function GetLinkInNumber(Link: TLink) : byte;
begin
  case Link of
    CTS: GetLinkInNumber:= $10;
    DSR: GetLinkInNumber:= $20;
    RI : GetLinkInNumber:= $40;
    DCD: GetLinkInNumber:= $80;
    DTR: GetLinkInNumber:= $01;
    RTS: GetLinkInNumber:= $02;
    TxD: GetLinkInNumber:= $40
  end;
end;

function ScanUserLink(Link: TLink): boolean;
begin
  ScanUserLink:= ReadPort(PBA + GetLinkOffset(Link)) and GetLinkInNumber(Link) = GetLinkInNumber(Link);
end;

end.
